@@ -7,22 +7,24 @@ mod macros;
 
 pub mod basic;
 #[cfg(not(Py_LIMITED_API))]
+#[cfg_attr(docsrs, doc(cfg(not(Py_LIMITED_API))))]
 pub mod buffer;
 pub mod context;
 pub mod descr;
 pub mod gc;
+#[doc(hidden)]
+pub mod impl_;
 pub mod iter;
 pub mod mapping;
 #[doc(hidden)]
 pub mod methods;
 pub mod number;
-#[doc(hidden)]
-pub mod proto_methods;
 pub mod pyasync;
 pub mod sequence;
 
 pub use self::basic::PyObjectProtocol;
 #[cfg(not(Py_LIMITED_API))]
+#[cfg_attr(docsrs, doc(cfg(not(Py_LIMITED_API))))]
 pub use self::buffer::PyBufferProtocol;
 pub use self::context::PyContextProtocol;
 pub use self::descr::PyDescrProtocol;
