@@ -110,7 +110,7 @@ impl PyErr {
     /// instance of that type.
     /// Otherwise, a `TypeError` is created instead.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// use pyo3::{Python, PyErr, IntoPy, exceptions::PyTypeError, types::PyType};
     /// Python::with_gil(|py| {
@@ -155,7 +155,7 @@ impl PyErr {
     ///
     /// The object will be normalized first if needed.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
     /// Python::with_gil(|py| {
@@ -171,7 +171,7 @@ impl PyErr {
     ///
     /// The object will be normalized first if needed.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
     /// Python::with_gil(|py| {
@@ -188,7 +188,7 @@ impl PyErr {
     ///
     /// The object will be normalized first if needed.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
     /// Python::with_gil(|py| {
@@ -375,7 +375,7 @@ impl PyErr {
 
     /// Clone the PyErr. This requires the GIL, which is why PyErr does not implement Clone.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// use pyo3::{Python, PyErr, exceptions::PyTypeError, types::PyType};
     /// Python::with_gil(|py| {
@@ -390,6 +390,28 @@ impl PyErr {
         PyErr::from_state(PyErrState::Normalized(self.normalized(py).clone()))
     }
 
+    /// Return the cause (either an exception instance, or None, set by `raise ... from ...`)
+    /// associated with the exception, as accessible from Python through `__cause__`.
+    pub fn cause(&self, py: Python) -> Option<PyErr> {
+        let ptr = unsafe { ffi::PyException_GetCause(self.pvalue(py).as_ptr()) };
+        let obj = unsafe { py.from_owned_ptr_or_opt::<PyAny>(ptr) };
+        obj.map(|x| Self::from_instance(x))
+    }
+
+    /// Set the cause associated with the exception, pass `None` to clear it.
+    pub fn set_cause(&self, py: Python, cause: Option<Self>) {
+        if let Some(cause) = cause {
+            let cause = cause.into_instance(py);
+            unsafe {
+                ffi::PyException_SetCause(self.pvalue(py).as_ptr(), cause.as_ptr());
+            }
+        } else {
+            unsafe {
+                ffi::PyException_SetCause(self.pvalue(py).as_ptr(), std::ptr::null_mut());
+            }
+        }
+    }
+
     fn from_state(state: PyErrState) -> PyErr {
         PyErr {
             state: UnsafeCell::new(Some(state)),
@@ -557,12 +579,6 @@ mod tests {
     #[test]
     #[should_panic(expected = "new panic")]
     fn fetching_panic_exception_resumes_unwind() {
-        // TODO replace with #[cfg(panic = "unwind")] once stable
-        if !crate::cfg_panic_unwind() {
-            // panic to meet the expected abort in panic=abort :-/
-            panic!("new panic");
-        }
-
         use crate::panic::PanicException;
 
         let gil = Python::acquire_gil();
@@ -633,4 +649,36 @@ mod tests {
         is_send::<PyErrState>();
         is_sync::<PyErrState>();
     }
+
+    #[test]
+    fn test_pyerr_cause() {
+        Python::with_gil(|py| {
+            let err = py
+                .run("raise Exception('banana')", None, None)
+                .expect_err("raising should have given us an error");
+            assert!(err.cause(py).is_none());
+
+            let err = py
+                .run(
+                    "raise Exception('banana') from Exception('apple')",
+                    None,
+                    None,
+                )
+                .expect_err("raising should have given us an error");
+            let cause = err
+                .cause(py)
+                .expect("raising from should have given us a cause");
+            assert_eq!(cause.to_string(), "Exception: apple");
+
+            err.set_cause(py, None);
+            assert!(err.cause(py).is_none());
+
+            let new_cause = exceptions::PyValueError::new_err("orange");
+            err.set_cause(py, Some(new_cause));
+            let cause = err
+                .cause(py)
+                .expect("set_cause should have given us a cause");
+            assert_eq!(cause.to_string(), "ValueError: orange");
+        });
+    }
 }
