@@ -5,7 +5,11 @@
 use crate::{ffi, internal_tricks::Unsendable, Python};
 use parking_lot::{const_mutex, Mutex, Once};
 use std::cell::{Cell, RefCell};
-use std::{mem::ManuallyDrop, ptr::NonNull};
+use std::{
+    mem::{self, ManuallyDrop},
+    ptr::NonNull,
+    sync::atomic,
+};
 
 static START: Once = Once::new();
 
@@ -45,12 +49,6 @@ pub(crate) fn gil_is_acquired() -> bool {
 /// If both the Python interpreter and Python threading are already initialized,
 /// this function has no effect.
 ///
-/// # Availability
-/// This function is only available when linking against Python distributions that contain a
-/// shared library.
-///
-/// This function is not available on PyPy.
-///
 /// # Panics
 /// - If the Python interpreter is initialized but Python threading is not,
 ///   a panic occurs.
@@ -58,7 +56,7 @@ pub(crate) fn gil_is_acquired() -> bool {
 ///   thread (the thread which originally initialized Python) also initializes
 ///   threading.
 ///
-/// # Example
+/// # Examples
 /// ```rust
 /// use pyo3::prelude::*;
 ///
@@ -70,8 +68,9 @@ pub(crate) fn gil_is_acquired() -> bool {
 ///     });
 /// }
 /// ```
-#[cfg(all(Py_SHARED, not(PyPy)))]
-#[allow(clippy::clippy::collapsible_if)] // for if cfg!
+#[cfg(not(PyPy))]
+#[cfg_attr(docsrs, doc(cfg(not(PyPy))))]
+#[allow(clippy::collapsible_if)] // for if cfg!
 pub fn prepare_freethreaded_python() {
     // Protect against race conditions when Python is not yet initialized and multiple threads
     // concurrently call 'prepare_freethreaded_python()'. Note that we do not protect against
@@ -109,12 +108,6 @@ pub fn prepare_freethreaded_python() {
 /// single process, it is not safe to call this function more than once. (Many such modules will not
 /// initialize correctly on the second run.)
 ///
-/// # Availability
-/// This function is only available when linking against Python distributions that contain a shared
-/// library.
-///
-/// This function is not available on PyPy.
-///
 /// # Panics
 /// - If the Python interpreter is already initalized before calling this function.
 ///
@@ -124,7 +117,7 @@ pub fn prepare_freethreaded_python() {
 /// - No Python APIs can be used after this function has finished executing.
 /// - The return value of the closure must not contain any Python value, _including_ `PyResult`.
 ///
-/// # Example
+/// # Examples
 /// ```rust
 /// use pyo3::prelude::*;
 ///
@@ -137,8 +130,9 @@ pub fn prepare_freethreaded_python() {
 ///     }
 /// }
 /// ```
-#[cfg(all(Py_SHARED, not(PyPy)))]
-#[allow(clippy::clippy::collapsible_if)] // for if cfg!
+#[cfg(not(PyPy))]
+#[cfg_attr(docsrs, doc(cfg(not(PyPy))))]
+#[allow(clippy::collapsible_if)] // for if cfg!
 pub unsafe fn with_embedded_python_interpreter<F, R>(f: F) -> R
 where
     F: for<'p> FnOnce(Python<'p>) -> R,
@@ -178,10 +172,13 @@ where
     result
 }
 
-/// RAII type that represents the Global Interpreter Lock acquisition. To get hold of a value of
-/// this type, see [`Python::acquire_gil`](struct.Python.html#method.acquire_gil).
+/// RAII type that represents the Global Interpreter Lock acquisition.
 ///
-/// # Example
+/// Users are strongly encouraged to [`Python::with_gil`](struct.Python.html#method.with_gil)
+/// instead of directly constructing this type.
+/// See [`Python::acquire_gil`](struct.Python.html#method.acquire_gil) for more.
+///
+/// # Examples
 /// ```
 /// use pyo3::Python;
 ///
@@ -190,6 +187,7 @@ where
 ///     let py = gil_guard.python();
 /// } // GIL is released when gil_guard is dropped
 /// ```
+#[allow(clippy::upper_case_acronyms)]
 #[must_use]
 pub struct GILGuard {
     gstate: ffi::PyGILState_STATE,
@@ -215,32 +213,9 @@ impl GILGuard {
         //    auto-initialize so this avoids breaking existing builds.
         //  - Otherwise, just check the GIL is initialized.
         cfg_if::cfg_if! {
-            if #[cfg(all(feature = "auto-initialize", Py_SHARED, not(PyPy)))] {
+            if #[cfg(all(feature = "auto-initialize", not(PyPy)))] {
                 prepare_freethreaded_python();
-            } else if #[cfg(all(feature = "auto-initialize", not(feature = "extension-module"), not(Py_SHARED), not(__pyo3_ci)))] {
-                compile_error!(concat!(
-                    "The `auto-initialize` feature is not supported when linking Python ",
-                    "statically instead of with a shared library.\n\n",
-                    "Please disable the `auto-initialize` feature, for example by entering the following ",
-                    "in your cargo.toml:\n\n",
-                    "    pyo3 = { version = \"",
-                    env!("CARGO_PKG_VERSION"),
-                    "\", default-features = false }\n\n",
-                    "Alternatively, compile PyO3 using a Python distribution which contains a shared ",
-                    "libary."
-                ));
-            } else if #[cfg(all(feature = "auto-initialize", not(feature = "extension-module"), PyPy, not(__pyo3_ci)))] {
-                compile_error!(concat!(
-                    "The `auto-initialize` feature is not supported by PyPy.\n\n",
-                    "Please disable the `auto-initialize` feature, for example by entering the following ",
-                    "in your cargo.toml:\n\n",
-                    "    pyo3 = { version = \"",
-                    env!("CARGO_PKG_VERSION"),
-                    "\", default-features = false }\n\n",
-                ));
             } else {
-                // extension module feature enabled and PyPy or static linking
-                // OR auto-initialize feature not enabled
                 START.call_once_force(|_| unsafe {
                     // Use call_once_force because if there is a panic because the interpreter is
                     // not initialized, it's fine for the user to initialize the interpreter and
@@ -248,12 +223,18 @@ impl GILGuard {
                     assert_ne!(
                         ffi::Py_IsInitialized(),
                         0,
-                        "The Python interpreter is not initalized and the `auto-initialize` feature is not enabled."
+                        "The Python interpreter is not initalized and the `auto-initialize` \
+                         feature is not enabled.\n\n\
+                         Consider calling `pyo3::prepare_freethreaded_python()` before attempting \
+                         to use Python APIs."
                     );
                     assert_ne!(
                         ffi::PyEval_ThreadsInitialized(),
                         0,
-                        "Python threading is not initalized and the `auto-initialize` feature is not enabled."
+                        "Python threading is not initalized and the `auto-initialize` feature is \
+                         not enabled.\n\n\
+                         Consider calling `pyo3::prepare_freethreaded_python()` before attempting \
+                         to use Python APIs."
                     );
                 });
             }
@@ -310,50 +291,51 @@ impl Drop for GILGuard {
     }
 }
 
+// Vector of PyObject(
+type PyObjVec = Vec<NonNull<ffi::PyObject>>;
+
 /// Thread-safe storage for objects which were inc_ref / dec_ref while the GIL was not held.
 struct ReferencePool {
-    pointers_to_incref: Mutex<Vec<NonNull<ffi::PyObject>>>,
-    pointers_to_decref: Mutex<Vec<NonNull<ffi::PyObject>>>,
+    dirty: atomic::AtomicBool,
+    // .0 is INCREFs, .1 is DECREFs
+    pointer_ops: Mutex<(PyObjVec, PyObjVec)>,
 }
 
 impl ReferencePool {
     const fn new() -> Self {
         Self {
-            pointers_to_incref: const_mutex(Vec::new()),
-            pointers_to_decref: const_mutex(Vec::new()),
+            dirty: atomic::AtomicBool::new(false),
+            pointer_ops: const_mutex((Vec::new(), Vec::new())),
         }
     }
 
     fn register_incref(&self, obj: NonNull<ffi::PyObject>) {
-        self.pointers_to_incref.lock().push(obj)
+        self.pointer_ops.lock().0.push(obj);
+        self.dirty.store(true, atomic::Ordering::Release);
     }
 
     fn register_decref(&self, obj: NonNull<ffi::PyObject>) {
-        self.pointers_to_decref.lock().push(obj)
+        self.pointer_ops.lock().1.push(obj);
+        self.dirty.store(true, atomic::Ordering::Release);
     }
 
     fn update_counts(&self, _py: Python) {
-        macro_rules! swap_vec_with_lock {
-            // Get vec from one of ReferencePool's mutexes via lock, swap vec if needed, unlock.
-            ($cell:expr) => {{
-                let mut locked = $cell.lock();
-                let mut out = Vec::new();
-                if !locked.is_empty() {
-                    std::mem::swap(&mut out, &mut *locked);
-                }
-                drop(locked);
-                out
-            }};
+        let prev = self.dirty.swap(false, atomic::Ordering::Acquire);
+        if !prev {
+            return;
         }
 
+        let mut ops = self.pointer_ops.lock();
+        let (increfs, decrefs) = mem::take(&mut *ops);
+        drop(ops);
         // Always increase reference counts first - as otherwise objects which have a
         // nonzero total reference count might be incorrectly dropped by Python during
         // this update.
-        for ptr in swap_vec_with_lock!(self.pointers_to_incref) {
+        for ptr in increfs {
             unsafe { ffi::Py_INCREF(ptr.as_ptr()) };
         }
 
-        for ptr in swap_vec_with_lock!(self.pointers_to_decref) {
+        for ptr in decrefs {
             unsafe { ffi::Py_DECREF(ptr.as_ptr()) };
         }
     }
@@ -364,6 +346,7 @@ unsafe impl Sync for ReferencePool {}
 static POOL: ReferencePool = ReferencePool::new();
 
 /// A RAII pool which PyO3 uses to store owned Python references.
+#[allow(clippy::upper_case_acronyms)]
 pub struct GILPool {
     /// Initial length of owned objects and anys.
     /// `Option` is used since TSL can be broken when `new` is called from `atexit`.
@@ -492,6 +475,7 @@ pub(crate) fn ensure_gil() -> EnsureGIL {
 }
 
 /// Struct used internally which avoids acquiring the GIL where it's not necessary.
+#[allow(clippy::upper_case_acronyms)]
 pub(crate) struct EnsureGIL(Option<GILGuard>);
 
 impl EnsureGIL {
@@ -774,15 +758,15 @@ mod test {
 
         // The pointer should appear once in the incref pool, and once in the
         // decref pool (for the clone being created and also dropped)
-        assert_eq!(&*POOL.pointers_to_incref.lock(), &vec![ptr]);
-        assert_eq!(&*POOL.pointers_to_decref.lock(), &vec![ptr]);
+        assert_eq!(&POOL.pointer_ops.lock().0, &vec![ptr]);
+        assert_eq!(&POOL.pointer_ops.lock().1, &vec![ptr]);
 
         // Re-acquring GIL will clear these pending changes
         drop(gil);
         let gil = Python::acquire_gil();
 
-        assert!(POOL.pointers_to_incref.lock().is_empty());
-        assert!(POOL.pointers_to_decref.lock().is_empty());
+        assert!(POOL.pointer_ops.lock().0.is_empty());
+        assert!(POOL.pointer_ops.lock().1.is_empty());
 
         // Overall count is still unchanged
         assert_eq!(count, obj.get_refcnt(gil.python()));
