@@ -3,7 +3,6 @@ use crate::conversion::{PyTryFrom, ToBorrowedObject};
 use crate::err::{PyDowncastError, PyErr, PyResult};
 use crate::gil;
 use crate::pycell::{PyBorrowError, PyBorrowMutError, PyCell};
-use crate::type_object::PyBorrowFlagLayout;
 use crate::types::{PyDict, PyTuple};
 use crate::{
     ffi, AsPyPointer, FromPyObject, IntoPy, IntoPyPointer, PyAny, PyClass, PyClassInitializer,
@@ -41,12 +40,12 @@ pub unsafe trait PyNativeType: Sized {
 ///  - [`Py::borrow`](#method.borrow), [`Py::try_borrow`](#method.try_borrow),
 ///    [`Py::borrow_mut`](#method.borrow_mut), or [`Py::try_borrow_mut`](#method.try_borrow_mut),
 ///    to directly access a `#[pyclass]` value (which has RefCell-like behavior, see
-///    [the `PyCell` guide entry](https://pyo3.rs/master/class.html#pycell-and-interior-mutability)
+///    [the `PyCell` guide entry](https://pyo3.rs/main/class.html#pycell-and-interior-mutability)
 ///    ).
 ///  - Use methods directly on `Py`, such as [`Py::call`](#method.call) and
 ///    [`Py::call_method`](#method.call_method).
 ///
-/// See [the guide](https://pyo3.rs/master/types.html) for an explanation
+/// See [the guide](https://pyo3.rs/main/types.html) for an explanation
 /// of the different Python object types.
 #[repr(transparent)]
 pub struct Py<T>(NonNull<ffi::PyObject>, PhantomData<T>);
@@ -59,10 +58,7 @@ where
     T: PyClass,
 {
     /// Create a new instance `Py<T>` of a `#[pyclass]` on the Python heap.
-    pub fn new(py: Python, value: impl Into<PyClassInitializer<T>>) -> PyResult<Py<T>>
-    where
-        T::BaseLayout: PyBorrowFlagLayout<T::BaseType>,
-    {
+    pub fn new(py: Python, value: impl Into<PyClassInitializer<T>>) -> PyResult<Py<T>> {
         let initializer = value.into();
         let obj = initializer.create_cell(py)?;
         let ob = unsafe { Py::from_owned_ptr(py, obj as _) };
@@ -113,7 +109,7 @@ where
     /// Python object reference in PyO3's object storage. The reference count for the Python
     /// object will not be decreased until the GIL lifetime ends.
     ///
-    /// # Example
+    /// # Examples
     ///
     /// Useful when returning GIL-bound references from functions. In the snippet below, note that
     /// the `'py` lifetime of the input GIL lifetime is also given to the returned reference:
@@ -358,19 +354,6 @@ impl<T> Py<T> {
         }
     }
 
-    /// Deprecated alias for [`from_owned_ptr`](#method.from_owned_ptr).
-    ///
-    /// # Safety
-    /// `ptr` must be a pointer to a Python object of type T.
-    ///
-    /// Callers must own the object referred to by `ptr`, as this function
-    /// implicitly takes ownership of that object.
-    #[inline]
-    #[deprecated = "this is a deprecated alias for Py::from_owned_ptr"]
-    pub unsafe fn from_owned_ptr_or_panic(py: Python, ptr: *mut ffi::PyObject) -> Py<T> {
-        Py::from_owned_ptr(py, ptr)
-    }
-
     /// Create a `Py<T>` instance by taking ownership of the given FFI pointer.
     ///
     /// If `ptr` is null then the current Python exception is fetched as a `PyErr`.
@@ -393,10 +376,7 @@ impl<T> Py<T> {
     /// If non-null, `ptr` must be a pointer to a Python object of type T.
     #[inline]
     pub unsafe fn from_owned_ptr_or_opt(_py: Python, ptr: *mut ffi::PyObject) -> Option<Self> {
-        match NonNull::new(ptr) {
-            Some(nonnull_ptr) => Some(Py(nonnull_ptr, PhantomData)),
-            None => None,
-        }
+        NonNull::new(ptr).map(|nonnull_ptr| Py(nonnull_ptr, PhantomData))
     }
 
     /// Create a `Py<T>` instance by creating a new reference from the given FFI pointer.
