@@ -1,11 +1,17 @@
 //! Includes `PyCell` implementation.
-use crate::conversion::{AsPyPointer, FromPyPointer, ToPyObject};
 use crate::exceptions::PyRuntimeError;
-use crate::pyclass::{PyClass, PyClassThreadChecker};
+use crate::pyclass::PyClass;
 use crate::pyclass_init::PyClassInitializer;
 use crate::pyclass_slots::{PyClassDict, PyClassWeakRef};
-use crate::type_object::{PyBorrowFlagLayout, PyLayout, PySizedLayout, PyTypeInfo};
+use crate::type_object::{PyLayout, PySizedLayout};
 use crate::types::PyAny;
+use crate::{class::impl_::PyClassBaseType, class::impl_::PyClassThreadChecker};
+use crate::{
+    conversion::{AsPyPointer, FromPyPointer, ToPyObject},
+    ffi::PyBaseObject_Type,
+    type_object::get_tp_free,
+    PyTypeInfo,
+};
 use crate::{ffi, IntoPy, PyErr, PyNativeType, PyObject, PyResult, Python};
 use std::cell::{Cell, UnsafeCell};
 use std::fmt;
@@ -16,82 +22,12 @@ use std::ops::{Deref, DerefMut};
 /// This is necessary for sharing BorrowFlag between parents and children.
 #[doc(hidden)]
 #[repr(C)]
-pub struct PyCellBase<T: PyTypeInfo> {
-    ob_base: T::Layout,
+pub struct PyCellBase<T> {
+    ob_base: T,
     borrow_flag: Cell<BorrowFlag>,
 }
 
-unsafe impl<T> PyLayout<T> for PyCellBase<T>
-where
-    T: PyTypeInfo + PyNativeType,
-    T::Layout: PySizedLayout<T>,
-{
-    const IS_NATIVE_TYPE: bool = true;
-}
-
-// Thes impls ensures `PyCellBase` can be a base type.
-impl<T> PySizedLayout<T> for PyCellBase<T>
-where
-    T: PyTypeInfo + PyNativeType,
-    T::Layout: PySizedLayout<T>,
-{
-}
-
-unsafe impl<T> PyBorrowFlagLayout<T> for PyCellBase<T>
-where
-    T: PyTypeInfo + PyNativeType,
-    T::Layout: PySizedLayout<T>,
-{
-}
-
-/// Inner type of `PyCell` without dict slots and reference counter.
-/// This struct has two usages:
-/// 1. As an inner type of `PyRef` and `PyRefMut`.
-/// 2. When `#[pyclass(extends=Base)]` is specified, `PyCellInner<Base>` is used as a base layout.
-#[doc(hidden)]
-#[repr(C)]
-pub struct PyCellInner<T: PyClass> {
-    ob_base: T::BaseLayout,
-    value: ManuallyDrop<UnsafeCell<T>>,
-}
-
-impl<T: PyClass> AsPyPointer for PyCellInner<T> {
-    fn as_ptr(&self) -> *mut ffi::PyObject {
-        (self as *const _) as *mut _
-    }
-}
-
-unsafe impl<T: PyClass> PyLayout<T> for PyCellInner<T> {
-    const IS_NATIVE_TYPE: bool = false;
-    fn get_super(&mut self) -> Option<&mut T::BaseLayout> {
-        Some(&mut self.ob_base)
-    }
-    fn py_init(&mut self, value: T) {
-        self.value = ManuallyDrop::new(UnsafeCell::new(value));
-    }
-    unsafe fn py_drop(&mut self, py: Python) {
-        ManuallyDrop::drop(&mut self.value);
-        self.ob_base.py_drop(py);
-    }
-}
-
-// These impls ensures `PyCellInner` can be a base type.
-impl<T: PyClass> PySizedLayout<T> for PyCellInner<T> {}
-unsafe impl<T: PyClass> PyBorrowFlagLayout<T> for PyCellInner<T> {}
-
-impl<T: PyClass> PyCellInner<T> {
-    fn get_ptr(&self) -> *mut T {
-        self.value.get()
-    }
-    fn get_borrow_flag(&self) -> BorrowFlag {
-        let base = (&self.ob_base) as *const _ as *const PyCellBase<T::BaseNativeType>;
-        unsafe { (*base).borrow_flag.get() }
-    }
-    fn set_borrow_flag(&self, flag: BorrowFlag) {
-        let base = (&self.ob_base) as *const _ as *const PyCellBase<T::BaseNativeType>;
-        unsafe { (*base).borrow_flag.set(flag) }
-    }
-}
+unsafe impl<T, U> PyLayout<T> for PyCellBase<U> where U: PySizedLayout<T> {}
 
 /// `PyCell` is the container type for [`PyClass`](../pyclass/trait.PyClass.html).
 ///
@@ -120,15 +56,15 @@ impl<T: PyClass> PyCellInner<T> {
 ///     name: &'static str,
 ///     author: &'static str,
 /// }
-/// let gil = Python::acquire_gil();
-/// let py = gil.python();
 /// let book = Book {
 ///     name: "The Man in the High Castle",
 ///     author: "Philip Kindred Dick",
 /// };
-/// let book_cell = PyCell::new(py, book).unwrap();
-/// // you can expose PyCell to Python snippets
-/// pyo3::py_run!(py, book_cell, "assert book_cell.name[-6:] == 'Castle'");
+/// Python::with_gil(|py| {
+///     let book_cell = PyCell::new(py, book).unwrap();
+///     // `&PyCell` implements `ToPyObject`, so you can use it in a Python snippet
+///     pyo3::py_run!(py, book_cell, "assert book_cell.name[-6:] == 'Castle'");
+/// });
 /// ```
 /// You can use `slf: &PyCell<Self>` as an alternative `self` receiver of `#[pymethod]`,
 /// though you rarely need it.
@@ -153,19 +89,25 @@ impl<T: PyClass> PyCellInner<T> {
 ///         Ok(*counter)
 ///     }
 /// }
-/// # let gil = Python::acquire_gil();
-/// # let py = gil.python();
-/// # let counter = PyCell::new(py, Counter::default()).unwrap();
-/// # pyo3::py_run!(py, counter, "assert counter.increment('cat') == 1");
+/// # Python::with_gil(|py| {
+/// #     let counter = PyCell::new(py, Counter::default()).unwrap();
+/// #     pyo3::py_run!(py, counter, "assert counter.increment('cat') == 1");
+/// # });
 /// ```
 #[repr(C)]
 pub struct PyCell<T: PyClass> {
-    inner: PyCellInner<T>,
-    thread_checker: T::ThreadChecker,
+    ob_base: <T::BaseType as PyClassBaseType>::LayoutAsBase,
+    contents: PyCellContents<T>,
+}
+
+#[repr(C)]
+pub(crate) struct PyCellContents<T: PyClass> {
+    pub(crate) value: ManuallyDrop<UnsafeCell<T>>,
+    pub(crate) thread_checker: T::ThreadChecker,
     // DO NOT CHANGE THE ORDER OF THESE FIELDS WITHOUT CHANGING PyCell::dict_offset()
     // AND PyCell::weakref_offset()
-    dict: T::Dict,
-    weakref: T::WeakRef,
+    pub(crate) dict: T::Dict,
+    pub(crate) weakref: T::WeakRef,
 }
 
 impl<T: PyClass> PyCell<T> {
@@ -202,10 +144,7 @@ impl<T: PyClass> PyCell<T> {
     /// In cases where the value in the cell does not need to be accessed immediately after
     /// creation, consider [`Py::new`](../instance/struct.Py.html#method.new) as a more efficient
     /// alternative.
-    pub fn new(py: Python, value: impl Into<PyClassInitializer<T>>) -> PyResult<&Self>
-    where
-        T::BaseLayout: PyBorrowFlagLayout<T::BaseType>,
-    {
+    pub fn new(py: Python, value: impl Into<PyClassInitializer<T>>) -> PyResult<&Self> {
         unsafe {
             let initializer = value.into();
             let self_ = initializer.create_cell(py)?;
@@ -244,27 +183,26 @@ impl<T: PyClass> PyCell<T> {
     /// # use pyo3::prelude::*;
     /// #[pyclass]
     /// struct Class {}
-    /// let gil = Python::acquire_gil();
-    /// let py = gil.python();
-    /// let c = PyCell::new(py, Class {}).unwrap();
-    /// {
-    ///     let m = c.borrow_mut();
-    ///     assert!(c.try_borrow().is_err());
-    /// }
+    /// Python::with_gil(|py| {
+    ///     let c = PyCell::new(py, Class {}).unwrap();
+    ///     {
+    ///         let m = c.borrow_mut();
+    ///         assert!(c.try_borrow().is_err());
+    ///     }
     ///
-    /// {
-    ///     let m = c.borrow();
-    ///     assert!(c.try_borrow().is_ok());
-    /// }
+    ///     {
+    ///         let m = c.borrow();
+    ///         assert!(c.try_borrow().is_ok());
+    ///     }
+    /// });
     /// ```
     pub fn try_borrow(&self) -> Result<PyRef<'_, T>, PyBorrowError> {
-        self.thread_checker.ensure();
-        let flag = self.inner.get_borrow_flag();
+        let flag = self.get_borrow_flag();
         if flag == BorrowFlag::HAS_MUTABLE_BORROW {
             Err(PyBorrowError { _private: () })
         } else {
-            self.inner.set_borrow_flag(flag.increment());
-            Ok(PyRef { inner: &self.inner })
+            self.set_borrow_flag(flag.increment());
+            Ok(PyRef { inner: self })
         }
     }
 
@@ -279,23 +217,22 @@ impl<T: PyClass> PyCell<T> {
     /// # use pyo3::prelude::*;
     /// #[pyclass]
     /// struct Class {}
-    /// let gil = Python::acquire_gil();
-    /// let py = gil.python();
-    /// let c = PyCell::new(py, Class {}).unwrap();
-    /// {
-    ///     let m = c.borrow();
-    ///     assert!(c.try_borrow_mut().is_err());
-    /// }
+    /// Python::with_gil(|py| {
+    ///     let c = PyCell::new(py, Class {}).unwrap();
+    ///     {
+    ///         let m = c.borrow();
+    ///         assert!(c.try_borrow_mut().is_err());
+    ///     }
     ///
-    /// assert!(c.try_borrow_mut().is_ok());
+    ///     assert!(c.try_borrow_mut().is_ok());
+    /// });
     /// ```
     pub fn try_borrow_mut(&self) -> Result<PyRefMut<'_, T>, PyBorrowMutError> {
-        self.thread_checker.ensure();
-        if self.inner.get_borrow_flag() != BorrowFlag::UNUSED {
+        if self.get_borrow_flag() != BorrowFlag::UNUSED {
             Err(PyBorrowMutError { _private: () })
         } else {
-            self.inner.set_borrow_flag(BorrowFlag::HAS_MUTABLE_BORROW);
-            Ok(PyRefMut { inner: &self.inner })
+            self.set_borrow_flag(BorrowFlag::HAS_MUTABLE_BORROW);
+            Ok(PyRefMut { inner: self })
         }
     }
 
@@ -314,26 +251,25 @@ impl<T: PyClass> PyCell<T> {
     /// # use pyo3::prelude::*;
     /// #[pyclass]
     /// struct Class {}
-    /// let gil = Python::acquire_gil();
-    /// let py = gil.python();
-    /// let c = PyCell::new(py, Class {}).unwrap();
+    /// Python::with_gil(|py| {
+    ///     let c = PyCell::new(py, Class {}).unwrap();
     ///
-    /// {
-    ///     let m = c.borrow_mut();
-    ///     assert!(unsafe { c.try_borrow_unguarded() }.is_err());
-    /// }
+    ///     {
+    ///         let m = c.borrow_mut();
+    ///         assert!(unsafe { c.try_borrow_unguarded() }.is_err());
+    ///     }
     ///
-    /// {
-    ///     let m = c.borrow();
-    ///     assert!(unsafe { c.try_borrow_unguarded() }.is_ok());
-    /// }
+    ///     {
+    ///         let m = c.borrow();
+    ///         assert!(unsafe { c.try_borrow_unguarded() }.is_ok());
+    ///     }
+    /// });
     /// ```
     pub unsafe fn try_borrow_unguarded(&self) -> Result<&T, PyBorrowError> {
-        self.thread_checker.ensure();
-        if self.inner.get_borrow_flag() == BorrowFlag::HAS_MUTABLE_BORROW {
+        if self.get_borrow_flag() == BorrowFlag::HAS_MUTABLE_BORROW {
             Err(PyBorrowError { _private: () })
         } else {
-            Ok(&*self.inner.value.get())
+            Ok(&*self.contents.value.get())
         }
     }
 
@@ -368,48 +304,17 @@ impl<T: PyClass> PyCell<T> {
         std::mem::swap(&mut *self.borrow_mut(), &mut *other.borrow_mut())
     }
 
-    /// Allocates a new PyCell given a type object `subtype`. Used by our `tp_new` implementation.
-    /// Requires `T::BaseLayout: PyBorrowFlagLayout<T::BaseType>` to ensure `self` has a borrow flag.
-    pub(crate) unsafe fn internal_new(
-        py: Python,
-        subtype: *mut ffi::PyTypeObject,
-    ) -> PyResult<*mut Self>
-    where
-        T::BaseLayout: PyBorrowFlagLayout<T::BaseType>,
-    {
-        let base = T::new(py, subtype);
-        if base.is_null() {
-            return Err(PyErr::fetch(py));
-        }
-        let base = base as *mut PyCellBase<T::BaseNativeType>;
-        (*base).borrow_flag = Cell::new(BorrowFlag::UNUSED);
-        let self_ = base as *mut Self;
-        (*self_).dict = T::Dict::new();
-        (*self_).weakref = T::WeakRef::new();
-        (*self_).thread_checker = T::ThreadChecker::new();
-        Ok(self_)
+    fn get_ptr(&self) -> *mut T {
+        self.contents.value.get()
     }
 }
 
-unsafe impl<T: PyClass> PyLayout<T> for PyCell<T> {
-    const IS_NATIVE_TYPE: bool = false;
-    fn get_super(&mut self) -> Option<&mut T::BaseLayout> {
-        Some(&mut self.inner.ob_base)
-    }
-    fn py_init(&mut self, value: T) {
-        self.inner.value = ManuallyDrop::new(UnsafeCell::new(value));
-    }
-    unsafe fn py_drop(&mut self, py: Python) {
-        ManuallyDrop::drop(&mut self.inner.value);
-        self.dict.clear_dict(py);
-        self.weakref.clear_weakrefs(self.as_ptr(), py);
-        self.inner.ob_base.py_drop(py);
-    }
-}
+unsafe impl<T: PyClass> PyLayout<T> for PyCell<T> {}
+impl<T: PyClass> PySizedLayout<T> for PyCell<T> {}
 
 impl<T: PyClass> AsPyPointer for PyCell<T> {
     fn as_ptr(&self) -> *mut ffi::PyObject {
-        self.inner.as_ptr()
+        (self as *const _) as *mut _
     }
 }
 
@@ -455,7 +360,7 @@ impl<T: PyClass + fmt::Debug> fmt::Debug for PyCell<T> {
 /// Wraps a borrowed reference to a value in a `PyCell<T>`.
 ///
 /// See the [`PyCell`](struct.PyCell.html) documentation for more.
-/// # Example
+/// # Examples
 /// You can use `PyRef` as an alternative of `&self` receiver when
 /// - You need to access the pointer of `PyCell`.
 /// - You want to get super class.
@@ -484,13 +389,13 @@ impl<T: PyClass + fmt::Debug> fmt::Debug for PyCell<T> {
 ///         format!("{}(base: {}, cnt: {})", slf.name, basename, refcnt)
 ///     }
 /// }
-/// # let gil = Python::acquire_gil();
-/// # let py = gil.python();
-/// # let sub = PyCell::new(py, Child::new()).unwrap();
-/// # pyo3::py_run!(py, sub, "assert sub.format() == 'Caterpillar(base: Butterfly, cnt: 3)'");
+/// # Python::with_gil(|py| {
+/// #     let sub = PyCell::new(py, Child::new()).unwrap();
+/// #     pyo3::py_run!(py, sub, "assert sub.format() == 'Caterpillar(base: Butterfly, cnt: 3)'");
+/// # });
 /// ```
 pub struct PyRef<'p, T: PyClass> {
-    inner: &'p PyCellInner<T>,
+    inner: &'p PyCell<T>,
 }
 
 impl<'p, T: PyClass> PyRef<'p, T> {
@@ -503,7 +408,7 @@ impl<'p, T: PyClass> PyRef<'p, T> {
 
 impl<'p, T, U> AsRef<U> for PyRef<'p, T>
 where
-    T: PyClass + PyTypeInfo<BaseType = U, BaseLayout = PyCellInner<U>>,
+    T: PyClass<BaseType = U>,
     U: PyClass,
 {
     fn as_ref(&self) -> &T::BaseType {
@@ -513,7 +418,7 @@ where
 
 impl<'p, T, U> PyRef<'p, T>
 where
-    T: PyClass + PyTypeInfo<BaseType = U, BaseLayout = PyCellInner<U>>,
+    T: PyClass<BaseType = U>,
     U: PyClass,
 {
     /// Get `PyRef<T::BaseType>`.
@@ -548,10 +453,10 @@ where
     ///         format!("{} {} {}", super_.as_ref().name1, super_.name2, subname)
     ///     }
     /// }
-    /// # let gil = Python::acquire_gil();
-    /// # let py = gil.python();
-    /// # let sub = PyCell::new(py, Sub::new()).unwrap();
-    /// # pyo3::py_run!(py, sub, "assert sub.name() == 'base1 base2 sub'")
+    /// # Python::with_gil(|py| {
+    /// #     let sub = PyCell::new(py, Sub::new()).unwrap();
+    /// #     pyo3::py_run!(py, sub, "assert sub.name() == 'base1 base2 sub'")
+    /// # });
     /// ```
     pub fn into_super(self) -> PyRef<'p, U> {
         let PyRef { inner } = self;
@@ -607,7 +512,7 @@ impl<T: PyClass + fmt::Debug> fmt::Debug for PyRef<'_, T> {
 ///
 /// See the [`PyCell`](struct.PyCell.html) and [`PyRef`](struct.PyRef.html) documentations for more.
 pub struct PyRefMut<'p, T: PyClass> {
-    inner: &'p PyCellInner<T>,
+    inner: &'p PyCell<T>,
 }
 
 impl<'p, T: PyClass> PyRefMut<'p, T> {
@@ -620,7 +525,7 @@ impl<'p, T: PyClass> PyRefMut<'p, T> {
 
 impl<'p, T, U> AsRef<U> for PyRefMut<'p, T>
 where
-    T: PyClass + PyTypeInfo<BaseType = U, BaseLayout = PyCellInner<U>>,
+    T: PyClass<BaseType = U>,
     U: PyClass,
 {
     fn as_ref(&self) -> &T::BaseType {
@@ -630,7 +535,7 @@ where
 
 impl<'p, T, U> AsMut<U> for PyRefMut<'p, T>
 where
-    T: PyClass + PyTypeInfo<BaseType = U, BaseLayout = PyCellInner<U>>,
+    T: PyClass<BaseType = U>,
     U: PyClass,
 {
     fn as_mut(&mut self) -> &mut T::BaseType {
@@ -640,7 +545,7 @@ where
 
 impl<'p, T, U> PyRefMut<'p, T>
 where
-    T: PyClass + PyTypeInfo<BaseType = U, BaseLayout = PyCellInner<U>>,
+    T: PyClass<BaseType = U>,
     U: PyClass,
 {
     /// Get `PyRef<T::BaseType>`.
@@ -701,11 +606,12 @@ impl<T: PyClass + fmt::Debug> fmt::Debug for PyRefMut<'_, T> {
     }
 }
 
+#[doc(hidden)]
 #[derive(Copy, Clone, Eq, PartialEq)]
-struct BorrowFlag(usize);
+pub struct BorrowFlag(usize);
 
 impl BorrowFlag {
-    const UNUSED: BorrowFlag = BorrowFlag(0);
+    pub(crate) const UNUSED: BorrowFlag = BorrowFlag(0);
     const HAS_MUTABLE_BORROW: BorrowFlag = BorrowFlag(usize::max_value());
     const fn increment(self) -> Self {
         Self(self.0 + 1)
@@ -764,3 +670,67 @@ impl From<PyBorrowMutError> for PyErr {
         PyRuntimeError::new_err(other.to_string())
     }
 }
+
+#[doc(hidden)]
+pub trait PyCellLayout<T>: PyLayout<T> {
+    fn get_borrow_flag(&self) -> BorrowFlag;
+    fn set_borrow_flag(&self, flag: BorrowFlag);
+    /// Implementation of tp_dealloc.
+    /// # Safety
+    /// - slf must be a valid pointer to an instance of a T or a subclass.
+    /// - slf must not be used after this call (as it will be freed).
+    unsafe fn tp_dealloc(slf: *mut ffi::PyObject, py: Python);
+}
+
+impl<T, U> PyCellLayout<T> for PyCellBase<U>
+where
+    U: PySizedLayout<T>,
+    T: PyTypeInfo,
+{
+    fn get_borrow_flag(&self) -> BorrowFlag {
+        self.borrow_flag.get()
+    }
+    fn set_borrow_flag(&self, flag: BorrowFlag) {
+        self.borrow_flag.set(flag)
+    }
+    unsafe fn tp_dealloc(slf: *mut ffi::PyObject, py: Python) {
+        // For `#[pyclass]` types which inherit from PyAny, we can just call tp_free
+        if T::type_object_raw(py) == &mut PyBaseObject_Type {
+            return get_tp_free(ffi::Py_TYPE(slf))(slf as _);
+        }
+
+        // More complex native types (e.g. `extends=PyDict`) require calling the base's dealloc.
+        #[cfg(not(Py_LIMITED_API))]
+        {
+            if let Some(dealloc) = (*T::type_object_raw(py)).tp_dealloc {
+                dealloc(slf as _);
+            } else {
+                get_tp_free(ffi::Py_TYPE(slf))(slf as _);
+            }
+        }
+
+        #[cfg(Py_LIMITED_API)]
+        unreachable!("subclassing native types is not possible with the `abi3` feature");
+    }
+}
+
+impl<T: PyClass> PyCellLayout<T> for PyCell<T>
+where
+    <T::BaseType as PyClassBaseType>::LayoutAsBase: PyCellLayout<T::BaseType>,
+{
+    fn get_borrow_flag(&self) -> BorrowFlag {
+        self.contents.thread_checker.ensure();
+        self.ob_base.get_borrow_flag()
+    }
+    fn set_borrow_flag(&self, flag: BorrowFlag) {
+        self.ob_base.set_borrow_flag(flag)
+    }
+    unsafe fn tp_dealloc(slf: *mut ffi::PyObject, py: Python) {
+        // Safety: Python only calls tp_dealloc when no references to the object remain.
+        let cell = &mut *(slf as *mut PyCell<T>);
+        ManuallyDrop::drop(&mut cell.contents.value);
+        cell.contents.dict.clear_dict(py);
+        cell.contents.weakref.clear_weakrefs(slf, py);
+        <T::BaseType as PyClassBaseType>::LayoutAsBase::tp_dealloc(slf, py)
+    }
+}
