@@ -140,7 +140,7 @@ impl Python<'_> {
     /// - If the `auto-initialize` feature is not enabled and the Python interpreter is not
     ///   initialized.
     ///
-    /// # Example
+    /// # Examples
     /// ```
     /// use pyo3::prelude::*;
     /// Python::with_gil(|py| -> PyResult<()> {
@@ -187,9 +187,9 @@ impl<'p> Python<'p> {
 
     /// Temporarily releases the `GIL`, thus allowing other Python threads to run.
     ///
-    /// # Example
+    /// # Examples
     /// ```
-    /// # use pyo3::prelude::*; use pyo3::types::IntoPyDict; use pyo3::wrap_pyfunction;
+    /// # use pyo3::prelude::*; use pyo3::types::IntoPyDict;
     /// use pyo3::exceptions::PyRuntimeError;
     /// use std::sync::Arc;
     /// use std::thread;
@@ -208,15 +208,16 @@ impl<'p> Python<'p> {
     ///         Ok(sum)
     ///     })
     /// }
-    /// let gil = Python::acquire_gil();
-    /// let py = gil.python();
-    /// let m = PyModule::new(py, "pcount").unwrap();
-    /// m.add_function(wrap_pyfunction!(parallel_count, m).unwrap()).unwrap();
-    /// let locals = [("pcount", m)].into_py_dict(py);
-    /// py.run(r#"
-    ///    s = ["Flow", "my", "tears", "the", "Policeman", "Said"]
-    ///    assert pcount.parallel_count(s, "a") == 3
-    /// "#, None, Some(locals));
+    ///
+    /// Python::with_gil(|py| {
+    ///     let m = PyModule::new(py, "pcount").unwrap();
+    ///     m.add_function(wrap_pyfunction!(parallel_count, m).unwrap()).unwrap();
+    ///     let locals = [("pcount", m)].into_py_dict(py);
+    ///     pyo3::py_run!(py, *locals, r#"
+    ///         s = ["Flow", "my", "tears", "the", "Policeman", "Said"]
+    ///         assert pcount.parallel_count(s, "a") == 3
+    ///     "#);
+    /// });
     /// ```
     ///
     /// **Note:**
@@ -234,7 +235,7 @@ impl<'p> Python<'p> {
     /// do much with those without a `Python<'p>` token, for which you'd need to
     /// reacquire the GIL.
     ///
-    /// # Example
+    /// # Examples
     /// ```compile_fail
     /// # use pyo3::prelude::*;
     /// # use pyo3::types::PyString;
@@ -277,14 +278,14 @@ impl<'p> Python<'p> {
     /// If `globals` is `None`, it defaults to Python module `__main__`.
     /// If `locals` is `None`, it defaults to the value of `globals`.
     ///
-    /// # Example:
+    /// # Examples
     /// ```
     /// # use pyo3::{types::{PyBytes, PyDict}, prelude::*};
-    /// # let gil = pyo3::Python::acquire_gil();
-    /// # let py = gil.python();
+    /// # Python::with_gil(|py| {
     /// let result = py.eval("[i * 10 for i in range(5)]", None, None).unwrap();
     /// let res: Vec<i64> = result.extract().unwrap();
     /// assert_eq!(res, vec![0, 10, 20, 30, 40])
+    /// # });
     /// ```
     pub fn eval(
         self,
@@ -300,25 +301,29 @@ impl<'p> Python<'p> {
     /// If `globals` is `None`, it defaults to Python module `__main__`.
     /// If `locals` is `None`, it defaults to the value of `globals`.
     ///
-    /// # Example:
+    /// # Examples
     /// ```
     /// use pyo3::{types::{PyBytes, PyDict}, prelude::*};
-    /// let gil = pyo3::Python::acquire_gil();
-    /// let py = gil.python();
-    /// let locals = PyDict::new(py);
-    /// py.run(
-    ///     r#"
+    /// Python::with_gil(|py| {
+    ///     let locals = PyDict::new(py);
+    ///     py.run(
+    ///         r#"
     /// import base64
     /// s = 'Hello Rust!'
     /// ret = base64.b64encode(s.encode('utf-8'))
     /// "#,
-    ///    None,
-    ///    Some(locals),
-    /// ).unwrap();
-    /// let ret = locals.get_item("ret").unwrap();
-    /// let b64: &PyBytes = ret.downcast().unwrap();
-    /// assert_eq!(b64.as_bytes(), b"SGVsbG8gUnVzdCE=");
+    ///       None,
+    ///       Some(locals),
+    ///     )
+    ///     .unwrap();
+    ///     let ret = locals.get_item("ret").unwrap();
+    ///     let b64: &PyBytes = ret.downcast().unwrap();
+    ///     assert_eq!(b64.as_bytes(), b"SGVsbG8gUnVzdCE=");
+    /// });
     /// ```
+    ///
+    /// You can use [`py_run!`](macro.py_run.html) for a handy alternative of `run`
+    /// if you don't need `globals` and unwrapping is OK.
     pub fn run(
         self,
         code: &str,
@@ -398,7 +403,7 @@ impl<'p> Python<'p> {
     ///
     /// This is a wrapper around the ffi call Py_GetVersion.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// # use pyo3::Python;
     /// Python::with_gil(|py| {
@@ -418,7 +423,7 @@ impl<'p> Python<'p> {
     /// Gets the running Python interpreter version as a struct similar to
     /// `sys.version_info`.
     ///
-    /// # Example
+    /// # Examples
     /// ```rust
     /// # use pyo3::Python;
     /// Python::with_gil(|py| {
@@ -551,62 +556,41 @@ impl<'p> Python<'p> {
         FromPyPointer::from_borrowed_ptr_or_opt(self, ptr)
     }
 
-    /// Checks whether `obj` is an instance of type `T`.
+    /// Lets the Python interpreter check and handle any pending signals. This will invoke the
+    /// corresponding signal handlers registered in Python (if any).
     ///
-    /// This is equivalent to the Python `isinstance` function.
-    #[deprecated(since = "0.13.0", note = "Please use obj.is_instance::<T>() instead")]
-    pub fn is_instance<T: PyTypeObject, V: AsPyPointer>(self, obj: &V) -> PyResult<bool> {
-        T::type_object(self).is_instance(obj)
-    }
-
-    /// Checks whether type `T` is subclass of type `U`.
+    /// Returns `Err(PyErr)` if any signal handler raises an exception.
     ///
-    /// This is equivalent to the Python `issubclass` function.
-    #[deprecated(
-        since = "0.13.0",
-        note = "Please use T::type_object(py).is_subclass::<U>() instead"
-    )]
-    pub fn is_subclass<T, U>(self) -> PyResult<bool>
-    where
-        T: PyTypeObject,
-        U: PyTypeObject,
-    {
-        T::type_object(self).is_subclass::<U>()
-    }
-
-    /// Releases a PyObject reference.
-    #[inline]
-    #[deprecated(since = "0.13.0", note = "Please just drop ob instead")]
-    pub fn release<T>(self, ob: T)
-    where
-        T: IntoPyPointer,
-    {
-        unsafe {
-            let ptr = ob.into_ptr();
-            if !ptr.is_null() {
-                ffi::Py_DECREF(ptr);
-            }
-        }
-    }
-
-    /// Releases a `ffi::PyObject` pointer.
-    #[inline]
-    #[deprecated(since = "0.13.0", note = "Please just drop obj instead")]
-    pub fn xdecref<T: IntoPyPointer>(self, ptr: T) {
-        unsafe { ffi::Py_XDECREF(ptr.into_ptr()) };
-    }
-
-    /// Lets the Python interpreter check for pending signals and invoke the
-    /// corresponding signal handlers. This can run arbitrary Python code.
+    /// These signals include `SIGINT` (normally raised by CTRL + C), which by default raises
+    /// `KeyboardInterrupt`. For this reason it is good practice to call this function regularly
+    /// as part of long-running Rust functions so that users can cancel it.
     ///
-    /// If an exception is raised by the signal handler, or the default signal
-    /// handler raises an exception (such as `KeyboardInterrupt` for `SIGINT`),
-    /// an `Err` is returned.
+    /// # Example
     ///
-    /// This is a wrapper of the C function `PyErr_CheckSignals()`. It is good
-    /// practice to call this regularly in a long-running calculation since
-    /// SIGINT and other signals handled by Python code are left pending for its
-    /// entire duration.
+    /// ```rust
+    /// use pyo3::prelude::*;
+    /// # fn main(){
+    /// #[pyfunction]
+    /// fn loop_forever(py: Python) -> PyResult<()> {
+    ///     loop {
+    ///         // As this loop is infinite it should check for signals every once in a while.
+    ///         // Using `?` causes any `PyErr` (potentially containing `KeyboardInterrupt`) to break out of the loop.
+    ///         py.check_signals()?;
+    ///
+    ///         // do work here
+    ///         # break Ok(()) // don't actually loop forever
+    ///     }
+    /// }
+    /// # }
+    /// ```
+    ///
+    /// # Note
+    /// This function calls [`PyErr_CheckSignals()`][1] which in turn may call signal handlers.
+    /// As Python's [`signal`][2] API allows users to define custom signal handlers, calling this
+    /// function allows arbitary Python code inside signal handlers to run.
+    ///
+    /// [1]: https://docs.python.org/3/c-api/exceptions.html?highlight=pyerr_checksignals#c.PyErr_CheckSignals
+    /// [2]: https://docs.python.org/3/library/signal.html
     pub fn check_signals(self) -> PyResult<()> {
         let v = unsafe { ffi::PyErr_CheckSignals() };
         if v == -1 {
@@ -644,27 +628,28 @@ impl<'p> Python<'p> {
     /// to use this API to clear memory, as PyO3 usually does not clear memory until the GIL is
     /// released.
     ///
-    /// # Example
+    /// # Examples
+    ///
     /// ```rust
     /// # use pyo3::prelude::*;
-    /// let gil = Python::acquire_gil();
-    /// let py = gil.python();
+    /// Python::with_gil(|py| {
+    ///     // Some long-running process like a webserver, which never releases the GIL.
+    ///     loop {
+    ///         // Create a new pool, so that PyO3 can clear memory at the end of the loop.
+    ///         let pool = unsafe { py.new_pool() };
     ///
-    /// // Some long-running process like a webserver, which never releases the GIL.
-    /// loop {
-    ///     // Create a new pool, so that PyO3 can clear memory at the end of the loop.
-    ///     let pool = unsafe { py.new_pool() };
+    ///         // It is recommended to *always* immediately set py to the pool's Python, to help
+    ///         // avoid creating references with invalid lifetimes.
+    ///         let py = unsafe { pool.python() };
     ///
-    ///     // It is recommended to *always* immediately set py to the pool's Python, to help
-    ///     // avoid creating references with invalid lifetimes.
-    ///     let py = unsafe { pool.python() };
-    ///
-    ///     // do stuff...
-    /// # break;  // Exit the loop so that doctest terminates!
-    /// }
+    ///         // do stuff...
+    /// #       break;  // Exit the loop so that doctest terminates!
+    ///     }
+    /// });
     /// ```
     ///
     /// # Safety
+    ///
     /// Extreme care must be taken when using this API, as misuse can lead to accessing invalid
     /// memory. In addition, the caller is responsible for guaranteeing that the GIL remains held
     /// for the entire lifetime of the returned `GILPool`.
@@ -691,7 +676,7 @@ impl<'p> Python<'p> {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::types::{IntoPyDict, PyAny, PyBool, PyInt, PyList};
+    use crate::types::{IntoPyDict, PyList};
 
     #[test]
     fn test_eval() {
@@ -734,35 +719,8 @@ mod test {
         assert_eq!(v, 2);
     }
 
-    #[test]
-    #[allow(deprecated)]
-    fn test_is_instance() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        assert!(py
-            .is_instance::<PyBool, PyAny>(PyBool::new(py, true).into())
-            .unwrap());
-        let list = PyList::new(py, &[1, 2, 3, 4]);
-        assert!(!py.is_instance::<PyBool, _>(list.as_ref()).unwrap());
-        assert!(py.is_instance::<PyList, _>(list.as_ref()).unwrap());
-    }
-
-    #[test]
-    #[allow(deprecated)]
-    fn test_is_subclass() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        assert!(py.is_subclass::<PyBool, PyInt>().unwrap());
-        assert!(!py.is_subclass::<PyBool, PyList>().unwrap());
-    }
-
     #[test]
     fn test_allow_threads_panics_safely() {
-        // TODO replace with #[cfg(panic = "unwind")] once stable
-        if !crate::cfg_panic_unwind() {
-            return;
-        }
-
         let gil = Python::acquire_gil();
         let py = gil.python();
 
