@@ -9,7 +9,7 @@ use std::slice;
 #[repr(transparent)]
 pub struct PyByteArray(PyAny);
 
-pyobject_native_var_type!(PyByteArray, ffi::PyByteArray_Type, ffi::PyByteArray_Check);
+pyobject_native_type_core!(PyByteArray, ffi::PyByteArray_Type, #checkfunction=ffi::PyByteArray_Check);
 
 impl PyByteArray {
     /// Creates a new Python bytearray object.
@@ -28,7 +28,7 @@ impl PyByteArray {
     /// * If `init` returns `Err(e)`, `new_with` will return `Err(e)`.
     /// * If `init` returns `Ok(())`, `new_with` will return `Ok(&PyByteArray)`.
     ///
-    /// # Example
+    /// # Examples
     /// ```
     /// use pyo3::{prelude::*, types::PyByteArray};
     /// Python::with_gil(|py| -> PyResult<()> {
@@ -128,15 +128,12 @@ impl PyByteArray {
 
     /// Copies the contents of the bytearray to a Rust vector.
     ///
-    /// # Example
+    /// # Examples
     ///
     /// ```
     /// # use pyo3::prelude::*;
     /// # use pyo3::types::PyByteArray;
-    /// # use pyo3::types::IntoPyDict;
-    /// # let gil = Python::acquire_gil();
-    /// # let py = gil.python();
-    /// #
+    /// # Python::with_gil(|py| {
     /// let bytearray = PyByteArray::new(py, b"Hello World.");
     /// let mut copied_message = bytearray.to_vec();
     /// assert_eq!(b"Hello World.", copied_message.as_slice());
@@ -144,8 +141,8 @@ impl PyByteArray {
     /// copied_message[11] = b'!';
     /// assert_eq!(b"Hello World!", copied_message.as_slice());
     ///
-    /// let locals = [("bytearray", bytearray)].into_py_dict(py);
-    /// py.run("assert bytearray == b'Hello World.'", None, Some(locals)).unwrap();
+    /// pyo3::py_run!(py, bytearray, "assert bytearray == b'Hello World.'");
+    /// # });
     /// ```
     pub fn to_vec(&self) -> Vec<u8> {
         unsafe { self.as_bytes() }.to_vec()
