@@ -20,7 +20,7 @@ pyobject_native_type!(
     PyDict,
     ffi::PyDictObject,
     ffi::PyDict_Type,
-    ffi::PyDict_Check
+    #checkfunction=ffi::PyDict_Check
 );
 
 impl PyDict {
@@ -37,6 +37,7 @@ impl PyDict {
     /// Returns an error on invalid input. In the case of key collisions,
     /// this keeps the last entry seen.
     #[cfg(not(PyPy))]
+    #[cfg_attr(docsrs, doc(cfg(not(PyPy))))]
     pub fn from_sequence(py: Python, seq: PyObject) -> PyResult<&PyDict> {
         unsafe {
             let dict = py.from_owned_ptr::<PyDict>(ffi::PyDict_New());
@@ -50,7 +51,7 @@ impl PyDict {
 
     /// Returns a new dictionary that contains the same key-value pairs as self.
     ///
-    /// This is equivalent to the Python expression `dict(self)`.
+    /// This is equivalent to the Python expression `self.copy()`.
     pub fn copy(&self) -> PyResult<&PyDict> {
         unsafe {
             self.py()
@@ -203,6 +204,15 @@ impl<'py> Iterator for PyDictIterator<'py> {
             }
         }
     }
+
+    #[inline]
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let len = self.dict.len().unwrap_or_default();
+        (
+            len.saturating_sub(self.pos as usize),
+            Some(len.saturating_sub(self.pos as usize)),
+        )
+    }
 }
 
 impl<'a> std::iter::IntoIterator for &'a PyDict {
@@ -527,8 +537,8 @@ mod test {
         v.insert(7, 32);
         let ob = v.to_object(py);
         let dict = <PyDict as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
-        assert_eq!(true, dict.contains(7i32).unwrap());
-        assert_eq!(false, dict.contains(8i32).unwrap());
+        assert!(dict.contains(7i32).unwrap());
+        assert!(!dict.contains(8i32).unwrap());
     }
 
     #[test]
@@ -697,6 +707,28 @@ mod test {
         assert_eq!(32 + 42 + 123, value_sum);
     }
 
+    #[test]
+    fn test_iter_size_hint() {
+        let gil = Python::acquire_gil();
+        let py = gil.python();
+        let mut v = HashMap::new();
+        v.insert(7, 32);
+        v.insert(8, 42);
+        v.insert(9, 123);
+        let ob = v.to_object(py);
+        let dict = <PyDict as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+
+        let mut iter = dict.iter();
+        assert_eq!(iter.size_hint(), (v.len(), Some(v.len())));
+        iter.next();
+        assert_eq!(iter.size_hint(), (v.len() - 1, Some(v.len() - 1)));
+
+        // Exhust iterator.
+        while let Some(_) = iter.next() {}
+
+        assert_eq!(iter.size_hint(), (0, Some(0)));
+    }
+
     #[test]
     fn test_into_iter() {
         let gil = Python::acquire_gil();
