@@ -1,7 +1,6 @@
 // Copyright (c) 2017-present PyO3 Project and Contributors
 
 use crate::err::{self, PyDowncastError, PyErr, PyResult};
-use crate::exceptions;
 use crate::ffi::{self, Py_ssize_t};
 use crate::instance::PyNativeType;
 use crate::types::{PyAny, PyList, PyTuple};
@@ -19,6 +18,7 @@ impl PySequence {
     ///
     /// This is equivalent to the Python expression `len(self)`.
     #[inline]
+    #[allow(clippy::len_without_is_empty)]
     pub fn len(&self) -> PyResult<isize> {
         let v = unsafe { ffi::PySequence_Size(self.as_ptr()) };
         if v == -1 {
@@ -188,6 +188,7 @@ impl PySequence {
     /// number of keys for which `self[key] == value`.
     #[inline]
     #[cfg(not(PyPy))]
+    #[cfg_attr(docsrs, doc(cfg(not(PyPy))))]
     pub fn count<V>(&self, value: V) -> PyResult<usize>
     where
         V: ToBorrowedObject,
@@ -257,57 +258,6 @@ impl PySequence {
     }
 }
 
-macro_rules! array_impls {
-    ($($N:expr),+) => {
-        $(
-            impl<'a, T> FromPyObject<'a> for [T; $N]
-            where
-                T: Copy + Default + FromPyObject<'a>,
-            {
-                #[cfg(not(feature = "nightly"))]
-                fn extract(obj: &'a PyAny) -> PyResult<Self> {
-                    let mut array = [T::default(); $N];
-                    extract_sequence_into_slice(obj, &mut array)?;
-                    Ok(array)
-                }
-
-                #[cfg(feature = "nightly")]
-                default fn extract(obj: &'a PyAny) -> PyResult<Self> {
-                    let mut array = [T::default(); $N];
-                    extract_sequence_into_slice(obj, &mut array)?;
-                    Ok(array)
-                }
-            }
-
-            #[cfg(feature = "nightly")]
-            impl<'source, T> FromPyObject<'source> for [T; $N]
-            where
-                for<'a> T: Default + FromPyObject<'a> + crate::buffer::Element,
-            {
-                fn extract(obj: &'source PyAny) -> PyResult<Self> {
-                    let mut array = [T::default(); $N];
-                    // first try buffer protocol
-                    if let Ok(buf) = crate::buffer::PyBuffer::get(obj) {
-                        if buf.dimensions() == 1 && buf.copy_to_slice(obj.py(), &mut array).is_ok() {
-                            buf.release(obj.py());
-                            return Ok(array);
-                        }
-                        buf.release(obj.py());
-                    }
-                    // fall back to sequence protocol
-                    extract_sequence_into_slice(obj, &mut array)?;
-                    Ok(array)
-                }
-            }
-        )+
-    }
-}
-
-array_impls!(
-    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
-    26, 27, 28, 29, 30, 31, 32
-);
-
 impl<'a, T> FromPyObject<'a> for Vec<T>
 where
     T: FromPyObject<'a>,
@@ -355,22 +305,6 @@ where
     Ok(v)
 }
 
-fn extract_sequence_into_slice<'s, T>(obj: &'s PyAny, slice: &mut [T]) -> PyResult<()>
-where
-    T: FromPyObject<'s>,
-{
-    let seq = <PySequence as PyTryFrom>::try_from(obj)?;
-    if seq.len()? as usize != slice.len() {
-        return Err(exceptions::PyBufferError::new_err(
-            "Slice length does not match buffer length.",
-        ));
-    }
-    for (value, item) in slice.iter_mut().zip(seq.iter()?) {
-        *value = item?.extract::<T>()?;
-    }
-    Ok(())
-}
-
 impl<'v> PyTryFrom<'v> for PySequence {
     fn try_from<V: Into<&'v PyAny>>(value: V) -> Result<&'v PySequence, PyDowncastError<'v>> {
         let value = value.into();
@@ -436,7 +370,7 @@ mod test {
         assert_eq!(0, seq.len().unwrap());
 
         let needle = 7i32.to_object(py);
-        assert_eq!(false, seq.contains(&needle).unwrap());
+        assert!(!seq.contains(&needle).unwrap());
     }
 
     #[test]
@@ -449,13 +383,13 @@ mod test {
         assert_eq!(6, seq.len().unwrap());
 
         let bad_needle = 7i32.to_object(py);
-        assert_eq!(false, seq.contains(&bad_needle).unwrap());
+        assert!(!seq.contains(&bad_needle).unwrap());
 
         let good_needle = 8i32.to_object(py);
-        assert_eq!(true, seq.contains(&good_needle).unwrap());
+        assert!(seq.contains(&good_needle).unwrap());
 
         let type_coerced_needle = 8f32.to_object(py);
-        assert_eq!(true, seq.contains(&type_coerced_needle).unwrap());
+        assert!(seq.contains(&type_coerced_needle).unwrap());
     }
 
     #[test]
@@ -593,10 +527,10 @@ mod test {
         let seq = ob.cast_as::<PySequence>(py).unwrap();
 
         let bad_needle = "blurst".to_object(py);
-        assert_eq!(false, seq.contains(bad_needle).unwrap());
+        assert!(!seq.contains(bad_needle).unwrap());
 
         let good_needle = "worst".to_object(py);
-        assert_eq!(true, seq.contains(good_needle).unwrap());
+        assert!(seq.contains(good_needle).unwrap());
     }
 
     #[test]
@@ -704,18 +638,6 @@ mod test {
         assert!(v == [1, 2, 3, 4]);
     }
 
-    #[test]
-    fn test_extract_bytearray_to_array() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let v: [u8; 3] = py
-            .eval("bytearray(b'abc')", None, None)
-            .unwrap()
-            .extract()
-            .unwrap();
-        assert!(&v == b"abc");
-    }
-
     #[test]
     fn test_extract_bytearray_to_vec() {
         let gil = Python::acquire_gil();
@@ -746,10 +668,10 @@ mod test {
         let py = gil.python();
         let list = vec![1].to_object(py);
         let seq = list.cast_as::<PySequence>(py).unwrap();
-        assert_eq!(seq.is_empty().unwrap(), false);
+        assert!(!seq.is_empty().unwrap());
         let vec: Vec<u32> = Vec::new();
         let empty_list = vec.to_object(py);
         let empty_seq = empty_list.cast_as::<PySequence>(py).unwrap();
-        assert_eq!(empty_seq.is_empty().unwrap(), true);
+        assert!(empty_seq.is_empty().unwrap());
     }
 }
