@@ -9,6 +9,12 @@ impl MyClass {
     fn class_attr_with_args(foo: i32) {}
 }
 
+#[pymethods]
+impl MyClass {
+    #[classattr(foobar)]
+    const CLASS_ATTR_WITH_ATTRIBUTE_ARG: i32 = 3;
+}
+
 #[pymethods]
 impl MyClass {
     fn staticmethod_without_attribute() {}
@@ -42,35 +48,35 @@ impl MyClass {
 #[pymethods]
 impl MyClass {
     #[new]
-    #[text_signature = "()"]
+    #[pyo3(text_signature = "()")]
     fn text_signature_on_new() {}
 }
 
 #[pymethods]
 impl MyClass {
     #[call]
-    #[text_signature = "()"]
+    #[pyo3(text_signature = "()")]
     fn text_signature_on_call(&self) {}
 }
 
 #[pymethods]
 impl MyClass {
     #[getter(x)]
-    #[text_signature = "()"]
+    #[pyo3(text_signature = "()")]
     fn text_signature_on_getter(&self) {}
 }
 
 #[pymethods]
 impl MyClass {
     #[setter(x)]
-    #[text_signature = "()"]
+    #[pyo3(text_signature = "()")]
     fn text_signature_on_setter(&self) {}
 }
 
 #[pymethods]
 impl MyClass {
     #[classattr]
-    #[text_signature = "()"]
+    #[pyo3(text_signature = "()")]
     fn text_signature_on_classattr() {}
 }
 
@@ -81,5 +87,31 @@ impl MyClass {
     fn multiple_method_types() {}
 }
 
+#[pymethods]
+impl MyClass {
+    fn generic_method<T>(value: T) {}
+}
+
+
+#[pymethods]
+impl MyClass {
+    fn impl_trait_method_first_arg(impl_trait: impl AsRef<PyAny>) {}
+}
+
+#[pymethods]
+impl MyClass {
+    fn impl_trait_method_second_arg(&self, impl_trait: impl AsRef<PyAny>) {}
+}
+
+#[pymethods]
+impl MyClass {
+    async fn async_method(&self) {}
+}
+
+#[pymethods]
+impl MyClass {
+    #[pyo3(pass_module)]
+    fn method_cannot_pass_module(&self, m: &PyModule) {}
+}
 
 fn main() {}
